<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2010 Frank Frewer <info@frankfrewer.de>
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

require_once(PATH_tslib.'class.tslib_pibase.php');


/**
 * Plugin 'Password Secured Download' for the 'fr_pwsdownload' extension.
 *
 * @author	Frank Frewer <info@frankfrewer.de>
 * @package	TYPO3
 * @subpackage	tx_frpwsdownload
 */
class tx_frpwsdownload_pi1 extends tslib_pibase {
	var $prefixId      = 'tx_frpwsdownload_pi1';		// Same as class name
	var $scriptRelPath = 'pi1/class.tx_frpwsdownload_pi1.php';	// Path to this script relative to the extension dir.
	var $extKey        = 'fr_pwsdownload';	// The extension key.
	var $pi_checkCHash = true;

	/**
	* Get data from FlexForm
	*/
	function getFlexFormData($field, $sheet='') {
		$result = "";
	
		if (empty($sheet)) {
			$result = $this->pi_getFFvalue($this->cObj->data['pi_flexform'], $field);
		}
		else {
			$result = $this->pi_getFFvalue($this->cObj->data['pi_flexform'], $field, $sheet);
		}
	
		return $result;
	}
	
	/**
	 * Displays the password - optionally the user - request or - if a valid password (and otionally a valid user) was inserted - a link for document download
	 *
	 * @param	string		$content: The PlugIn content
	 * @param	array		$conf: The PlugIn configuration
	 * @return	The content that is displayed on the website
	 */
	function main($content,$conf)	{

		$this->pi_initPIflexForm();
		$this->conf=$conf;
		$this->pi_setPiVarDefaults();
		$this->pi_loadLL();
		$this->now = time();
		
		//set some values
		$pid = $this->cObj->data['pid'];
		$uid = $this->cObj->data['uid'];
		$pathUploads = t3lib_div::dirname($_SERVER['SCRIPT_FILENAME']).'/uploads/tx_frpwsdownload/';
		$pluginPath = t3lib_extMgm::siteRelPath($this->extKey);

		//get data from flexform
		$document = $this->getFlexFormData('document', 'sDEF');
		$title = $this->getFlexFormData('title', 'sDEF');
		if (empty($title)) { $title = $document; }
		$allowedCode = $this->getFlexFormData('code', 'sDEF');
		$allowedUsers = $this->getFlexFormData('users', 'sDEF');

		//  Get the template
		$this->templateCode = $this->cObj->fileResource($this->conf["templateFile"]);
 
    		// Get the parts out of the template
    		$t = array();
    		$t["getCodeForm"] = $this->cObj->getSubpart($this->templateCode, "###TEMPLATE_GET_CODE_FORM###");
    		$t["downloadLink"] = $this->cObj->getSubpart($this->templateCode, "###TEMPLATE_DOWNLOAD_LINK###");
    		$t["error"] = $this->cObj->getSubpart($this->templateCode, "###TEMPLATE_ERROR###");

		$markerArray = array();
		$markerArray['###TITLE###'] = $title;

		if (empty($allowedCode)) { 
			// no password was set. Show only the error and exit. Don't give a download possibility.
			$markerArray['###ERROR###'] = $this->pi_getLL('errorNoPasswordSet');
			$content = $this->cObj->substituteMarkerArrayCached($t["error"], $markerArray, array(), array());
			return $this->pi_wrapInBaseClass($content);
		}

		if (!empty($allowedUsers)) {

			// get allowed users in an array 
			$allowedUsersArray = explode(",", $allowedUsers);
//			foreach ($allowedUsersArray as &$value) {
			foreach ($allowedUsersArray as $value) {
				$value = trim($value);
			}
			unset($value); // break the reference with the last element
		}

		$reqUid = $this->piVars['uid'];
		if ($this->piVars['action'] != "" && $reqUid == $uid) {
			$action = $this->piVars['action'];
			$givenCode = $this->piVars['code'];
			$givenUser = $this->piVars['user'];
		}
		else {
			$action = "getCode";
		}


		switch ($action) {
		case "checkCode":
			$userMatched = true;
			// there can be an user field input only if $allowedUsers is not empty
			if (!empty($allowedUsersArray)){
				if (!in_array($givenUser, $allowedUsersArray)) {
					// the given user is wrong
					$userMatched = false;
				}
			}
			if ( $allowedCode == addslashes($givenCode) && $userMatched == true) {
				// fill an array with statistic data
				$statisticFields = array();
				$statisticFields['element_uid'] = $uid;
				$statisticFields['element_pid'] = $pid;
				$statisticFields['element_title'] = $title;
				$statisticFields['element_document'] = $document;
				$statisticFields['element_password'] = $allowedCode;
				$statisticFields['element_user'] = $givenUser;
				$path_info = pathinfo($pathUploads.$document);
				$statisticFields['element_type'] = $path_info['extension'];
				$statisticFields['element_mtime'] = date ("Y-m-d H:i:s", filemtime($pathUploads.$document));
				$statisticFields['year'] = date('Y');
				$statisticFields['month'] = date('m');
				$statisticFields['day'] = date('d');
				$statisticFields['tstamp'] = $this->now;
				$statisticFields['crdate'] = $this->now;
				$statisticFields['counter'] = 1;

				// insert a record in the statistic table
				$res = $GLOBALS['TYPO3_DB']->exec_INSERTquery('tx_frpwsdownload_statdata', $statisticFields);
				unset($statisticFields);

				//set data in session for download.php
				session_start();
				$_SESSION[$this->prefixId]['file'] = $pathUploads . $document; 
				$_SESSION[$this->prefixId]['title'] = $document;

				// create content
				$markerArray['###START_DOWNLOAD###'] = $this->pi_getLL('startDownload');
				$markerArray['###DOWNLOAD_URL###'] = $pluginPath . 'download.php';
				$markerArray['###DOWNLOAD_TEXT###'] = $this->pi_getLL('downloadText');

				$content .= $this->cObj->substituteMarkerArrayCached($t["downloadLink"], $markerArray, array(), array());
	
				break;
			}
			elseif (!empty($allowedUsersArray)) { 
				// the given user is wrong
				$markerArray['###ERROR###'] = $this->pi_getLL('errorWrongPasswordOrUser');
				$content = $this->cObj->substituteMarkerArrayCached($t["error"], $markerArray, array(), array());
				break;

			}
			else { 
				// the given password is wrong
				$markerArray['###ERROR###'] = $this->pi_getLL('errorWrongPassword');
				$content = $this->cObj->substituteMarkerArrayCached($t["error"], $markerArray, array(), array());
				break;

			}

		case "getCode":
			if (!empty($document)) {
				// create content
				$markerArray['###ACTION_URL###'] = $this->pi_getPageLink($pid) . '?' . $this->prefixId.'[action]=checkCode';
				$markerArray['###PROMPT###'] = (empty($allowedUsersArray) ?	 $this->pi_getLL('promptPasswordOnly') : $this->pi_getLL('promptUserAndPassword')) . '&nbsp;';

				if (!empty($allowedUsersArray)){
					// it is requiered to input a user 
					$markerArray['###LABEL_USER###'] = $this->pi_getLL('labelUser');
					$markerArray['###INPUT_USER###'] = '<input type="text" name="' . $this->prefixId . '[user]" value="" />';
				} 
				else {
					$markerArray['###LABEL_USER###'] = '';
					$markerArray['###INPUT_USER###'] = '';
				}
				$markerArray['###LABEL_PASSWORD###'] = $this->pi_getLL('labelPassword');
				$markerArray['###INPUT_PASSWORD###'] = '<input type="text" name="' . $this->prefixId . '[code]" value="" />';
				$markerArray['###UID###'] = $uid;
				$markerArray['###FORM_SUBMIT###'] = $this->pi_getLL('submitButton');
				$content .= $this->cObj->substituteMarkerArrayCached($t["getCodeForm"], $markerArray, array(), array());

			} 
			else {
				// there is no document defined
				$markerArray['###ERROR###'] = $this->pi_getLL('errorNoDocument');
				$content = $this->cObj->substituteMarkerArrayCached($t["error"], $markerArray, array(), array());
				break;

			}
			break;
		}

		return $this->pi_wrapInBaseClass($content);
	}

	/**
	 * Updates the statistic table
	 *
	 * @param	string		$content: The PlugIn content
	 * @param	array		$conf: The PlugIn configuration
	 * @return	The content that is displayed on the website
	 */
	function updateStatisticsTable($content,$conf)	{
		// not used at the moment

	}
}



if (defined('TYPO3_MODE') && $TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/fr_pwsdownload/pi1/class.tx_frpwsdownload_pi1.php'])	{
	include_once($TYPO3_CONF_VARS[TYPO3_MODE]['XCLASS']['ext/fr_pwsdownload/pi1/class.tx_frpwsdownload_pi1.php']);
}

?>